/*
 * SharedHash.h
 *
 * Shared memory hash table with long term disk storage.
 *
 * Copyright 2001, 2002 by Anthony Howe.  All rights reserved.
 */

#ifndef __com_snert_mod_watch_SharedHash_h__
#define __com_snert_mod_watch_SharedHash_h__	1

/***********************************************************************
 *** No configuration below this point.
 ***********************************************************************/

#include <sys/types.h>

/*
 * This is the data that will be kept in shared memory.
 */
struct shEntry {
	char *key;
	unsigned long ifInOctets;
	unsigned long ifOutOctets;
	unsigned long ifRequests;
	unsigned long ifDocuments;
	short         ifActive;
	double	      ifOutRate;
	unsigned long periodOctets;
	unsigned long periodMarker;
};

/*
 * This statistical information will be kept in shared memory.
 */
struct shInfo {
	unsigned long found;		/* found in hash table */
	unsigned long probes;		/* number of extra probes made */
	unsigned long faults;		/* swapped to disk */
	unsigned long flushes;		/* out of shared memory */
};

/*
 * This will be kept in conventional memory.
 */
struct shTable {
	void *mutex;
	void *memory;			/* Memory header. */
	void *shared;			/* Start of shared memory block. */
	void *eshared;			/* First byte beyond the end of block.*/
	char *pathname;			/* Workdir + NAME_MAX space */
	uid_t uid;			/* User ID of a weenie file. */
	gid_t gid;			/* Group ID of a weenie file. */
	const char *workdir;		/* The working directory. */
	struct shInfo *info;		/* Statistic information. */
	struct shEntry *array;
	int size;			/* Allocated number of entries. */
};

#ifdef __cplusplus
extern "C" {
#endif

extern const char shLockFile[];
extern const char shScanFormat[];
extern const char shPrintFormat[];

extern int shLock(struct shTable *table);
extern int shUnlock(struct shTable *table);
extern void shFlushAll(struct shTable *table);
extern unsigned long shHashCode(unsigned long hash, const char *k);
extern int shContainsKey(struct shTable *table, const char *key);
extern struct shEntry *shGetLockedEntry(struct shTable *table, const char *key);
extern struct shTable *shCreate(struct pool *p, int size, const char *workdir, uid_t uid, gid_t gid);
extern void shDestroy(void *data);

#ifdef  __cplusplus
}
#endif

#endif /* __com_snert_mod_watch_SharedHash_h__ */
